/* Resolving Segfaults in Legacy Intel® MPI Library on Newer Linux* Distributions
 *
 * https://software.intel.com/content/www/us/en/develop/articles/resolving-segfaults-in-legacy-intel-mpi-library-on-newer-linux-distributions.html
 *
 * Compile the file by 
 *   gcc -c -Wall -Werror -fpic ./strtok_proxy.c
 *   gcc -ldl -shared -o ./strtok_proxy.so ./strtok_proxy.o
 * And apply the generated library at runtime using the following:
 *   export LD_PRELOAD=/full_path_to/strtok_proxy.so
 *
 * */

#define _GNU_SOURCE 1

#include <string.h>
#include <dlfcn.h>

typedef char *(*strtok_t) (char *str, const char *delimiters);
static strtok_t strtok_orig_p = NULL;

char *strtok(char *str, const char *delimiters)
{

    if (strtok_orig_p == NULL) {
        strtok_orig_p = (strtok_t) dlsym(RTLD_NEXT, "strtok");
        if (strtok_orig_p == NULL) {
            /* Error handling */
            return NULL;
        }
        if (str == NULL) {
            str = "";
        }
    }

    return strtok_orig_p(str, delimiters);
}

